﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Windows.Threading;

namespace Praktikum_2_Loesung
{
    public partial class MainWindow : Window
    {
        public const int Breite = 40;
        public const int Höhe = 25; 
        Zellenzustand[,] zellenzustände = new Zellenzustand[Höhe, Breite]; // Spalten und Zeilen gerne "falsch" herum, ist hier aber nicht so wichtig
        DispatcherTimer timer = new DispatcherTimer();
        Mähroboter[] dieMähroboter = new Mähroboter[4];

        public MainWindow()
        {
            InitializeComponent();

            timer.Interval = TimeSpan.FromSeconds(0.2);
            timer.Tick += Timer_Tick;
        }

        private void Timer_Tick(object sender, EventArgs e)
        {
            for (int i = 0; i < dieMähroboter.Length; i++)
            {
                int x = dieMähroboter[i].X;
                int y = dieMähroboter[i].Y;
                zellenzustände[y, x] = Zellenzustand.Gemäht;
                dieMähroboter[i].MacheEinenSchritt(zellenzustände[y - 1, x], zellenzustände[y, x + 1], zellenzustände[y + 1, x], zellenzustände[y, x - 1]);

                x = dieMähroboter[i].X;
                y = dieMähroboter[i].Y;
                zellenzustände[y, x] = Zellenzustand.Roboter;
            }

            Zeichenfläche.Children.Clear(); // Ineffizient! Lieber die bestehenden Rechtecke umfärben.

            for (int y = 0; y < Höhe; y++)
            {
                for (int x = 0; x < Breite; x++)
                {
                    Brush brush = null;
                    switch (zellenzustände[y, x])
                    {
                        case Zellenzustand.Ungemäht:
                            brush = Brushes.Green;
                            break;
                        case Zellenzustand.Gemäht:
                            brush = Brushes.LightGreen;
                            break;
                        case Zellenzustand.Verboten:
                            brush = Brushes.LightPink;
                            break;
                        case Zellenzustand.Roboter:
                            brush = Brushes.Red;
                            break;
                    }

                    Rectangle rectangle = new Rectangle();
                    rectangle.Width = Zeichenfläche.Width / Breite;
                    rectangle.Height = Zeichenfläche.Height / Höhe;
                    rectangle.Fill = brush;
                    Zeichenfläche.Children.Add(rectangle);
                    Canvas.SetLeft(rectangle, x * rectangle.Width);
                    Canvas.SetTop(rectangle, y * rectangle.Height);
                }
            }
        }

        private void buttonStart_Click(object sender, RoutedEventArgs e)
        {
            for (int y = 0; y < Höhe; y++)
            {
                for (int x = 0; x < Breite; x++)
                {
                    bool istAmRand = (x == 0 || x == Breite - 1 || y == 0 || y == Höhe - 1);
                    zellenzustände[y, x] = istAmRand ? Zellenzustand.Verboten : Zellenzustand.Ungemäht;
                }
            }

            Random random = new Random();

            for (int i = 0; i < 3; i++)
            {
                int aussparungLinks = random.Next(Breite);
                int aussparungBreite = random.Next(Breite - aussparungLinks + 1);
                int aussparungOben = random.Next(Höhe);
                int aussparungHöhe = random.Next(Höhe - aussparungOben + 1);

                for (int y = aussparungOben; y < aussparungOben + aussparungHöhe; y++)
                {
                    for (int x = aussparungLinks; x < aussparungLinks + aussparungBreite; x++)
                    {
                        zellenzustände[y, x] = Zellenzustand.Verboten;
                    }
                }
            }

            for (int i = 0; i < dieMähroboter.Length; i++)
            {
                while(true)
                {
                    int x = random.Next(Breite);
                    int y = random.Next(Höhe);
                    if(zellenzustände[y, x] == Zellenzustand.Ungemäht)
                    {
                        zellenzustände[y, x] = Zellenzustand.Roboter;
                        dieMähroboter[i] = new Mähroboter(x, y);
                        break;
                    }
                }
            }

            timer.Start(); // läuft ggf. schon
        }
    }
}
